TRANSACTION_TRANSFER = 0
TRANSACTION_SECOND_SIGNATURE_REGISTRATION = 1
TRANSACTION_DELEGATE_REGISTRATION = 2
TRANSACTION_VOTE = 3
TRANSACTION_MULTI_SIGNATURE_REGISTRATION = 4
TRANSACTION_IPFS = 5
TRANSACTION_TIMELOCK_TRANSFER = 6
TRANSACTION_MULTI_PAYMENT = 7
TRANSACTION_DELEGATE_RESIGNATION = 8

TRANSACTION_TYPES = {
    TRANSACTION_TRANSFER: 'transfer',
    TRANSACTION_SECOND_SIGNATURE_REGISTRATION: 'second_signature_registration',
    TRANSACTION_DELEGATE_REGISTRATION: 'delegate_registration',
    TRANSACTION_VOTE: 'vote',
    TRANSACTION_MULTI_SIGNATURE_REGISTRATION: 'multi_signature_registration',
    TRANSACTION_IPFS: 'ipfs',
    TRANSACTION_TIMELOCK_TRANSFER: 'timelock_transfer',
    TRANSACTION_MULTI_PAYMENT: 'multi_payment',
    TRANSACTION_DELEGATE_RESIGNATION: 'delegate_resignation',
}

TRANSACTION_FEES = {
    TRANSACTION_TRANSFER: 10000000,
    TRANSACTION_SECOND_SIGNATURE_REGISTRATION: 500000000,
    TRANSACTION_DELEGATE_REGISTRATION: 2500000000,
    TRANSACTION_VOTE: 100000000,
    TRANSACTION_MULTI_SIGNATURE_REGISTRATION: 500000000,
    TRANSACTION_IPFS: 0,
    TRANSACTION_TIMELOCK_TRANSFER: 0,
    TRANSACTION_MULTI_PAYMENT: 0,
    TRANSACTION_DELEGATE_RESIGNATION: 0,
}
